@@ -28,4 +28,20 @@ if [ "$res" != "0" ];
 	then echo "vdbcache test FAILED, res=$res output=$output" && exit 1;
 fi
 
-echo vdbcache test is finished
\ No newline at end of file
+echo vdbcache test is finished
+
+# ncbi_phid in error message:
+output=$(NCBI_SETTINGS=../LIBS-GUID.mkfg ${bin_dir}/${srapath} qq 2>&1)
+
+res=$?
+if [ "$res" == "0" ];
+	then echo "error test FAILED, res=$res output=$output" && exit 1;
+fi
+
+grep -q "ncbi_phid=" <<< "$output" > /dev/null
+res=$?
+if [ "$res" != "0" ];
+	then echo "error test FAILED (no "ncbi_phid" found), res=$res output=$output" && exit 1;
+fi
+
+echo error test is finished
\ No newline at end of file
