@@ -1,4 +1,4 @@
-/*==============================================================================
+/*=============================================================================$
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -24,6 +24,14 @@
 *
 */
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE /* memchr */
+#endif
+
+#ifndef _DEFAULT_SOURCE
+#define _DEFAULT_SOURCE /* strncasecmp */
+#endif
+
 #include "configure.h"
 
 #include <cloud/manager.h> /* CloudMgrRelease */
@@ -200,6 +208,12 @@ static const char* USAGE_P_CW[] = { "Prefetch downloads to "
 static const char* USAGE_P_UR[] = { "Prefetch downloads to "
     "public user repository when it is set (default).", NULL };
 
+#define ALIAS_TEL "T"
+#define OPTION_TEL "enable-telemetry"
+static const char* USAGE_TEL[] = {
+    "Report limited tools usage information back to SRA (default: yes).",
+    NULL };
+
 rc_t WorkspaceDirPathConv(const Args * args, uint32_t arg_index, const char * arg, size_t arg_len, void ** result, WhackParamFnP * whack)
 {
     uint32_t imp_count = 0;
@@ -245,6 +259,7 @@ OptDef Options[] =
     , { OPTION_S3_C,ALIAS_S3_C,NULL, USAGE_S3_C,1, true , false, NULL }
     , { OPTION_S3_F,ALIAS_S3_F,NULL, USAGE_S3_F,1, true , false, NULL }
     , { OPTION_S3_P,ALIAS_S3_P,NULL, USAGE_S3_P,1, true , false, NULL }
+    , { OPTION_TEL, ALIAS_TEL, NULL, USAGE_TEL, 1, true , false, NULL }
 };
 
 ParamDef Parameters[] =
@@ -308,6 +323,10 @@ rc_t CC Usage(const Args* args) {
     HelpOptionLine(ALIAS_P_CW, OPTION_P_CW, NULL, USAGE_P_CW);
     HelpOptionLine(ALIAS_P_UR, OPTION_P_UR, NULL, USAGE_P_UR);
     KOutMsg("\n");
+#ifdef TELEMETRY
+    HelpOptionLine(ALIAS_TEL , OPTION_TEL , "yes | no", USAGE_TEL);
+    KOutMsg("\n");
+#endif
     HelpOptionLine (ALIAS_PRX, OPTION_PRX, "uri[:port]", USAGE_PRX);
     HelpOptionLine (ALIAS_PRD, OPTION_PRD, "yes | no", USAGE_PRD);
     KOutMsg ("\n");
@@ -540,6 +559,8 @@ typedef struct Params {
 
     EState prefetchToCwd;
     EState preferNoQuality;
+
+    EState disableTelemetry;
 } Params;
 
 static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
@@ -991,6 +1012,7 @@ static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
                 prm->modeCloud = true;
             }
         }
+/********************************* cloud end **********************************/
 /* OPTION_P_UR */
         {
             rc = ArgsOptionCount(args, OPTION_P_UR, &pcount);
@@ -1017,7 +1039,28 @@ static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
                 prm->modeCloud = true;
             }
         }
-/********************************* cloud end **********************************/
+/* OPTION_TEL */
+        {
+            rc = ArgsOptionCount(args, OPTION_TEL, &pcount);
+            if (rc != 0) {
+                LOGERR(klogErr, rc,
+                    "Failure to get '" OPTION_TEL "' argument");
+                break;
+            }
+            if (pcount > 0) {
+                rc = ArgsOptionValue(args, OPTION_TEL, 0,
+                    (const void **)&dummy);
+                if (rc != 0) {
+                    LOGERR(klogErr, rc,
+                        "Failure to get '" OPTION_TEL "' argument");
+                    break;
+                }
+                if (strcasecmp(dummy, "no") == 0)
+                    prm->disableTelemetry = eTrue;
+                else
+                    prm->disableTelemetry = eFalse;
+            }
+        }
    /* OPTION_ALL */
         {
             rc = ArgsOptionCount(args, OPTION_ALL, &pcount);
@@ -1037,6 +1080,7 @@ static rc_t ParamsConstruct(int argc, char* argv[], Params* prm) {
                     && !prm->modeShowModules
                     && !prm->modeSetNode
                     && !prm->preferNoQuality
+                    && !prm->disableTelemetry
                     && prm->ngc == NULL
                     && prm->proxy == NULL && prm->proxyDisabled == eUndefined))
                 /* show all by default */
@@ -1142,7 +1186,7 @@ static rc_t In(const char* prompt, const char* def, char** read) {
                 string_copy_measure(buf, sizeof buf, def);
             }
             if (buf[0]) {
-                *read = strdup(buf);
+                *read = string_dup_measure(buf, NULL);
                 if (*read == NULL) {
                     rc = RC
                         (rcExe, rcStorage, rcAllocating, rcMemory, rcExhausted);
@@ -1380,7 +1424,7 @@ static rc_t SetNode(KConfig* cfg, const Params* prm) {
             "normally this application should not be run as root/superuser");
     }
 
-    name = strdup(prm->setValue);
+    name = string_dup_measure(prm->setValue, NULL);
     if (name == NULL)
     {   return RC(rcExe, rcStorage, rcAllocating, rcMemory, rcExhausted); }
 
@@ -1923,6 +1967,19 @@ rc_t S3SetAcceptCharges(KConfig * cfg, EState value, bool * set) {
     return rc;
 }
 
+static const char * s_ToAbsPath(const char * value) {
+    static char reslvd[4096] = "";
+    KDirectory * d = NULL;
+    rc_t rc = KDirectoryNativeDir(&d);
+    if (rc == 0) {
+        rc = KDirectoryResolvePath(d, true, reslvd, sizeof reslvd, "%s", value);
+        if (rc == 0)
+            value = reslvd;
+        KDirectoryRelease(d);
+    }
+    return value;
+}
+
 static rc_t S3SetCredentialsFile(KConfig * cfg, const char * aValue,
     bool * set)
 {
@@ -1932,8 +1989,14 @@ static rc_t S3SetCredentialsFile(KConfig * cfg, const char * aValue,
         return rc;
     if (value[0] == ' ' && value[1] == '\0')
         value = "";
+    else
+        value = s_ToAbsPath(value);
     rc = KConfig_Set_Aws_Credential_File(cfg, value);
     if (rc == 0) {
+        char buf[PATH_MAX] = "";
+        rc_t rc = KConfig_Get_Aws_Credential_File(cfg, buf, sizeof buf, NULL);
+        if (rc == 0)
+            value = buf;
         assert(set);
         *set = true;
         OUTMSG(("Path to AWS Credentials File was set to '%s'\n", value));
@@ -1984,8 +2047,14 @@ static rc_t GsSetCredentialsFile(KConfig * cfg, const char * aValue,
         return rc;
     if (value[0] == ' ' && value[1] == '\0')
         value = "";
+    else
+        value = s_ToAbsPath(value);
     rc = KConfig_Set_Gcp_Credential_File(cfg, value);
     if (rc == 0) {
+        char buf[PATH_MAX] = "";
+        rc_t rc = KConfig_Get_Gcp_Credential_File(cfg, buf, sizeof buf, NULL);
+        if (rc == 0)
+            value = buf;
         assert(set);
         *set = true;
         OUTMSG(("Path to GCP Credentials File was set to '%s'\n", value));
@@ -2301,6 +2370,18 @@ rc_t CC KMain(int argc, char* argv[]) {
             if (rc == 0 && r != 0)
                 rc = r;
         }
+        
+        if (prm.disableTelemetry) {
+            rc_t r = KConfig_Set_SendTelemetry
+                (cfg, prm.disableTelemetry == eFalse);
+            if (rc == 0 && r != 0)
+                rc = r;
+            if (rc == 0) {
+                rc_t r2 = KConfigCommit(cfg);
+                if (r2 != 0)
+                    rc = r2;
+            }
+        }
     }
 
     if (prm.modeShowEnv)
