@@ -75,6 +75,7 @@ Global Options:
 * options effecting performance optimisation
   tmpfs <directory>                 where to store temparary files, default: '/tmp'
   cache-size <mbytes>               the limit in MB for temparary files
+  min-batch-size <number>           minimum batch size for spot assembly (default 10e6)
 
 * options effecting error limits
   max-err-count <number>            the maximum number of errors to ignore
@@ -104,6 +105,7 @@ Filtering Options:
 Rare or Esoteric Options:
   input <directory>                 where to find fasta files
   ref-file <file>                   fasta file with references
+  force-sorted                      disable sort-order checking
   unsorted                          expect unsorted input (requires more memory)
   sorted                            require sorted input
   TI                                look for trace id optional tag
@@ -132,6 +134,7 @@ static char const option_config[] = "config";
 static char const option_min_mapq[] = "min-mapq";
 static char const option_qual_compress[] = "qual-quant";
 static char const option_cache_size[] = "cache-size";
+static char const option_no_sort_check[] = "force-sorted";
 static char const option_unsorted[] = "unsorted";
 static char const option_sorted[] = "sorted";
 static char const option_max_err_count[] = "max-err-count";
@@ -161,6 +164,7 @@ static char const option_defer_secondary[] = "defer-secondary";
 static char const option_spot_batch_size[] = "batch-size";
 static char const option_threads[] = "threads";
 static char const option_extra_logging[] = "extra-logging";
+static char const option_min_batch_size[] = "min-batch-size";
 
 #define OPTION_INPUT option_input
 #define OPTION_OUTPUT option_output
@@ -189,6 +193,7 @@ static char const option_extra_logging[] = "extra-logging";
 #define OPTION_SPOT_BATCH_SIZE option_spot_batch_size
 #define OPTION_THREADS option_threads
 #define OPTION_EXTRA_LOGGING option_extra_logging
+#define OPTION_MIN_BATCH_SIZE option_min_batch_size
 
 #define ALIAS_INPUT  "i"
 #define ALIAS_OUTPUT "o"
@@ -248,6 +253,13 @@ char const * qcomp_usage[] =
     NULL
 };
 
+static
+char const * no_sort_usage[] =
+{
+    "Disable sort order checking.",
+    NULL
+};
+
 static
 char const * unsorted_usage[] =
 {
@@ -459,12 +471,19 @@ char const * is_extra_logging[] =
     NULL
 };
 
+static
+char const * min_batch_size_usage[] =
+{
+    "Set the minimum batch size for spot assembly (default: 10,000,000 spots)",
+    NULL
+};
+
 OptDef Options[] =
 {
     /* order here is same as in param array below!!! */
-    { OPTION_INPUT, ALIAS_INPUT,  NULL,  input_usage, 1, true,  false },
-    { OPTION_OUTPUT, ALIAS_OUTPUT, NULL, output_usage, 1, true,  true },
-    { OPTION_CONFIG, ALIAS_CONFIG,  NULL,  config_usage, 1, true, false },
+    { OPTION_INPUT, ALIAS_INPUT, NULL, input_usage, 1, true, false },
+    { OPTION_OUTPUT, ALIAS_OUTPUT, NULL, output_usage, 1, true, true },
+    { OPTION_CONFIG, ALIAS_CONFIG, NULL, config_usage, 1, true, false },
     { OPTION_HEADER, NULL, NULL, use_header, 1, true, false },
     { OPTION_TMPFS, ALIAS_TMPFS, NULL, tmpfs_usage, 1, true,  false },
     { OPTION_UNALIGNED, ALIAS_UNALIGNED, NULL, use_unaligned, 256, true, false },
@@ -477,17 +496,18 @@ OptDef Options[] =
     { OPTION_NO_CS, NULL, NULL, use_no_cs, 1, false,  false },
     { OPTION_MIN_MATCH, NULL, NULL, use_min_match, 1, true, false },
     { OPTION_NO_SECONDARY, ALIAS_NO_SECONDARY, NULL, use_no_secondary, 1, false, false },
-    { option_unsorted, NULL, NULL, unsorted_usage, 1, false,  false },
-    { option_sorted, NULL, NULL, sorted_usage, 1, false,  false },
-    { option_no_verify, NULL, NULL, no_verify_usage, 1, false,  false },
-    { option_only_verify, NULL, NULL, only_verify_usage, 1, false,  false },
-    { option_use_qual, NULL, NULL, use_QUAL_usage, 1, false,  false },
-    { option_ref_config, NULL, NULL, use_ref_config, 1, false,  false },
-    { option_ref_filter, NULL, NULL, use_ref_filter, 1, true,  false },
+    { option_no_sort_check, NULL, NULL, no_sort_usage, 1, false, false },
+    { option_unsorted, NULL, NULL, unsorted_usage, 1, false, false },
+    { option_sorted, NULL, NULL, sorted_usage, 1, false, false },
+    { option_no_verify, NULL, NULL, no_verify_usage, 1, false, false },
+    { option_only_verify, NULL, NULL, only_verify_usage, 1, false, false },
+    { option_use_qual, NULL, NULL, use_QUAL_usage, 1, false, false },
+    { option_ref_config, NULL, NULL, use_ref_config, 1, false, false },
+    { option_ref_filter, NULL, NULL, use_ref_filter, 1, true, false },
     { option_edit_aligned_qual, NULL, NULL, use_edit_aligned_qual, 1, true, false },
-    { option_keep_mismatch_qual, NULL, NULL, use_keep_mismatch_qual, 1, false,  false },
-    { OPTION_MAX_REC_COUNT, NULL, NULL, mrc_usage, 1, true,  false },
-    { OPTION_MAX_ERR_COUNT, ALIAS_MAX_ERR_COUNT, NULL, mec_usage, 1, true,  false },
+    { option_keep_mismatch_qual, NULL, NULL, use_keep_mismatch_qual, 1, false, false },
+    { OPTION_MAX_REC_COUNT, NULL, NULL, mrc_usage, 1, true, false },
+    { OPTION_MAX_ERR_COUNT, ALIAS_MAX_ERR_COUNT, NULL, mec_usage, 1, true, false },
     { OPTION_REF_FILE, ALIAS_REF_FILE, NULL, use_ref_file, 0, true, false },
     { OPTION_TI, NULL, NULL, use_TI, 1, false, false },
     { OPTION_MAX_WARN_DUP_FLAG, NULL, NULL, use_max_dup_warnings, 1, true, false },
@@ -497,7 +517,8 @@ OptDef Options[] =
     { OPTION_DEFER_SECONDARY, NULL, NULL, use_defer_secondary, 1, false, false },
     { OPTION_SPOT_BATCH_SIZE, NULL, NULL, spot_batch_size, 1, true, false },
     { OPTION_THREADS, NULL, NULL, number_of_threads, 1, true, false },
-    { OPTION_EXTRA_LOGGING, NULL, NULL, is_extra_logging, 1, false, false }
+    { OPTION_EXTRA_LOGGING, NULL, NULL, is_extra_logging, 1, false, false },
+    { OPTION_MIN_BATCH_SIZE, NULL, NULL, min_batch_size_usage, 1, true,  false },
 };
 
 const char* OptHelpParam[] =
@@ -538,7 +559,8 @@ const char* OptHelpParam[] =
     NULL,				/* defer secondary */
     NULL,				/* search batch size */
     NULL,				/* threads */
-    NULL				/* extra logging */
+    NULL,				/* extra logging */
+    "count"			    /* min cache size */
 };
 
 rc_t UsageSummary (char const * progname)
@@ -558,11 +580,12 @@ rc_t UsageSummary (char const * progname)
 
 char const UsageDefaultName[] = "bam-load";
 static const unsigned DEFAULT_BATCH_SIZE = 240e6;
+static const unsigned DEFAULT_MIN_SPOT_ASSEMPLY_BATCH_SIZE = 10e6;
 
 rc_t CC Usage (const Args * args)
 {
     rc_t rc;
-    int i;
+    size_t i;
     const char * progname = UsageDefaultName;
     const char * fullpath = UsageDefaultName;
     const size_t argsQty = sizeof(Options) / sizeof(Options[0]);
@@ -706,11 +729,19 @@ static rc_t getArgValue(Args *const args, char const *name, int index, char cons
     rc_t const rc = ArgsOptionValue(args, name, index, &value);
     if (rc) return rc;
     free((void *)*result);
-    *result = strdup((const char*)value);
+    *result = string_dup_measure((const char*)value, NULL);
     assert(*result);
     return 0;
 }
 
+static bool getFlag(Args *const args, char const *name, rc_t *rc)
+{
+    uint32_t count = 0;
+    *rc = ArgsOptionCount(args, name, &count);
+    return *rc == 0 ? (count > 0) : false;
+}
+
+#define SET_FLAG(FLAG, OPTNAME) FLAG |= getFlag(args, OPTNAME, &rc); if (rc) break
 static rc_t main_1(int argc, char *argv[], bool const continuing, unsigned const load)
 {
     Args *args;
@@ -729,25 +760,24 @@ static rc_t main_1(int argc, char *argv[], bool const continuing, unsigned const
     while (rc == 0) {
         uint32_t pcount;
 
-        rc = ArgsOptionCount(args, option_only_verify, &pcount);
-        if (rc)
-            break;
-        G.onlyVerifyReferences |= (pcount > 0);
-
-        rc = ArgsOptionCount(args, option_no_verify, &pcount);
-        if (rc)
-            break;
-        G.noVerifyReferences |= (pcount > 0);
-
-        rc = ArgsOptionCount(args, option_use_qual, &pcount);
-        if (rc)
-            break;
-        G.useQUAL |= (pcount > 0);
-
-        rc = ArgsOptionCount(args, option_ref_config, &pcount);
-        if (rc)
-            break;
-        G.limit2config |= (pcount > 0);
+        SET_FLAG(G.onlyVerifyReferences, option_only_verify);
+        SET_FLAG(G.noVerifyReferences, option_no_verify);
+        SET_FLAG(G.useQUAL, option_use_qual);
+        SET_FLAG(G.limit2config, option_ref_config);
+        SET_FLAG(G.noSortOrderCheck, option_no_sort_check);
+        SET_FLAG(G.expectUnsorted, option_unsorted);
+        SET_FLAG(G.requireSorted, option_sorted);
+        SET_FLAG(G.acceptBadDups, OPTION_ACCEPT_DUP);
+        SET_FLAG(G.acceptNoMatch, OPTION_ACCEPT_NOMATCH);
+        SET_FLAG(G.keepMismatchQual, option_keep_mismatch_qual);
+        SET_FLAG(G.noColorSpace, OPTION_NO_CS);
+        SET_FLAG(G.noSecondary, OPTION_NO_SECONDARY);
+        SET_FLAG(G.hasTI, OPTION_TI);
+        SET_FLAG(G.acceptHardClip, OPTION_ACCEPT_HARD_CLIP);
+        SET_FLAG(G.allowMultiMapping, OPTION_ALLOW_MULTI_MAP);
+        SET_FLAG(G.assembleWithSecondary, OPTION_ALLOW_SECONDARY);
+        SET_FLAG(G.deferSecondary, OPTION_DEFER_SECONDARY);
+        SET_FLAG(G.hasExtraLogging, OPTION_EXTRA_LOGGING);
 
         rc = ArgsOptionCount(args, OPTION_REF_FILE, &pcount);
         if (rc)
@@ -847,7 +877,7 @@ static rc_t main_1(int argc, char *argv[], bool const continuing, unsigned const
             if (rc)
                 break;
             if (load == 0) {
-                G.firstOut = strdup(G.outpath);
+                G.firstOut = string_dup_measure(G.outpath, NULL);
             }
             value = strrchr(G.outpath, '/');
             G.outname = value ? (value + 1) : G.outpath;
@@ -933,16 +963,6 @@ static rc_t main_1(int argc, char *argv[], bool const continuing, unsigned const
             G.maxWarnCount_DupConflict = strtoul(value, &dummy, 0);
         }
 
-        rc = ArgsOptionCount (args, option_unsorted, &pcount);
-        if (rc)
-            break;
-        G.expectUnsorted |= (pcount > 0);
-
-        rc = ArgsOptionCount (args, option_sorted, &pcount);
-        if (rc)
-            break;
-        G.requireSorted |= (pcount > 0);
-
         rc = ArgsOptionCount (args, OPTION_MAX_REC_COUNT, &pcount);
         if (rc)
             break;
@@ -976,56 +996,6 @@ static rc_t main_1(int argc, char *argv[], bool const continuing, unsigned const
             G.minMatchCount = strtoul(value, &dummy, 0);
         }
 
-        rc = ArgsOptionCount (args, OPTION_ACCEPT_DUP, &pcount);
-        if (rc)
-            break;
-        G.acceptBadDups |= (pcount > 0);
-
-        rc = ArgsOptionCount (args, OPTION_ACCEPT_NOMATCH, &pcount);
-        if (rc)
-            break;
-        G.acceptNoMatch |= (pcount > 0);
-
-        rc = ArgsOptionCount (args, option_keep_mismatch_qual, &pcount);
-        if (rc)
-            break;
-        G.keepMismatchQual |= (pcount > 0);
-
-        rc = ArgsOptionCount (args, OPTION_NO_CS, &pcount);
-        if (rc)
-            break;
-        G.noColorSpace |= (pcount > 0);
-
-        rc = ArgsOptionCount (args, OPTION_NO_SECONDARY, &pcount);
-        if (rc)
-            break;
-        G.noSecondary |= (pcount > 0);
-
-        rc = ArgsOptionCount (args, OPTION_TI, &pcount);
-        if (rc)
-            break;
-        G.hasTI |= (pcount > 0);
-
-        rc = ArgsOptionCount (args, OPTION_ACCEPT_HARD_CLIP, &pcount);
-        if (rc)
-            break;
-        G.acceptHardClip |= (pcount > 0);
-
-        rc = ArgsOptionCount (args, OPTION_ALLOW_MULTI_MAP, &pcount);
-        if (rc)
-            break;
-        G.allowMultiMapping |= (pcount > 0);
-
-        rc = ArgsOptionCount (args, OPTION_ALLOW_SECONDARY, &pcount);
-        if (rc)
-            break;
-        G.assembleWithSecondary |= (pcount > 0);
-
-        rc = ArgsOptionCount (args, OPTION_DEFER_SECONDARY, &pcount);
-        if (rc)
-            break;
-        G.deferSecondary |= (pcount > 0);
-
         rc = ArgsOptionCount (args, OPTION_NOMATCH_LOG, &pcount);
         if (rc)
             break;
@@ -1164,11 +1134,22 @@ static rc_t main_1(int argc, char *argv[], bool const continuing, unsigned const
                 G.numThreads = 8;
         }
 
-        rc = ArgsOptionCount (args, OPTION_EXTRA_LOGGING, &pcount);
+        rc = ArgsOptionCount (args, OPTION_MIN_BATCH_SIZE, &pcount);
         if (rc)
             break;
-        G.hasExtraLogging |= (pcount > 0);
-
+        if (pcount == 1)
+        {
+            rc = ArgsOptionValue (args, OPTION_MIN_BATCH_SIZE, 0, (const void **)&value);
+            if (rc)
+                break;
+            G.minBatchSize = strtoul(value, &dummy, 0);
+            if (G.minBatchSize == 0) {
+                rc = RC(rcApp, rcArgv, rcAccessing, rcParam, rcIncorrect);
+                OUTMSG (("min-batch-size: bad value\n"));
+                MiniUsage (args);
+                break;
+            }
+        }
 
         rc = run(argv[0], n_aligned, (char const **)aligned, n_unalgnd, (char const **)unalgnd, continuing);
         break;
@@ -1185,6 +1166,7 @@ static rc_t main_1(int argc, char *argv[], bool const continuing, unsigned const
     ArgsWhack(args);
     return rc;
 }
+#undef SET_FLAG
 
 static void cleanupGlobal(void)
 {
@@ -1301,12 +1283,12 @@ rc_t CC KMain(int argc, char *argv[])
 
     if (has_help) {
         argc = 2;
-        argv[1] = "--help";
+        argv[1] = (char*)"--help";
         return main_help_vers(argc, argv);
     }
     if (has_vers) {
         argc = 2;
-        argv[1] = "--version";
+        argv[1] = (char*)"--version";
         return main_help_vers(argc, argv);
     }
 
@@ -1316,16 +1298,17 @@ rc_t CC KMain(int argc, char *argv[])
     G.mode = mode_Archive;
     G.globalMode = mode_Archive;
     G.maxSeqLen = TableWriterRefSeq_MAX_SEQ_LEN;
-    G.schemaPath = strdup(SCHEMAFILE);
+    G.schemaPath = string_dup_measure(SCHEMAFILE, NULL);
     G.omit_aligned_reads = true;
     G.omit_reference_reads = true;
     G.minMapQual = 0; /* accept all */
-    G.tmpfs = strdup("/tmp");
+    G.tmpfs = string_dup_measure("/tmp", NULL);
     G.cache_size = ((size_t)16) << 30;
     G.maxErrCount = 1000;
     G.minMatchCount = 10;
     G.searchBatchSize = DEFAULT_BATCH_SIZE;
     G.numThreads = 8;
+    G.minBatchSize = DEFAULT_MIN_SPOT_ASSEMPLY_BATCH_SIZE;
     set_pid();
 
     for (arglast = 1; arglast < argc; ++arglast) {
